import os

from API import LabFloorImporterPlugin
from API import LabFloorItem
from NL.CommonConcepts.Configurations.MoleculeConfiguration import MoleculeConfiguration
from NL.CommonConcepts.PeriodicTable import SYMBOL_TO_ELEMENT
from NL.CommonConcepts.PhysicalQuantity import Angstrom
from NL.ComputerScienceUtilities import Exceptions

from XYZFileRawReader import XYZFileRawReader


class XYZLabFloor(LabFloorImporterPlugin):
    """
    Class for handling the imports of XYZ-files as LabFloor items.
    """

    def scan(self, filename):
        """
        Scans a file to check if it is supported by the plugin

        @param filename : The path to be scanned.
        @type           : string

        @return A list of LabFloorItems
        """
        # Setup a resulting vector.
        result = []

        # Determine extension
        basename = os.path.basename(filename)
        no_extension_name, extension = os.path.splitext(basename)

        # Return empty string if extension isn't ".xyz"
        if extension != '.xyz':
            return result

        # Try to load configuration
        try:
            reader = XYZFileRawReader(filename)
        except Exception:
            return result

        for molecule_idx in xrange(reader.numOfMolecules()):

            # Read the comment for this molecule.
            comment = reader.comment(molecule=molecule_idx)

            # Create and add LabFloorItem to list
            if reader.numOfMolecules() == 1:
                title = no_extension_name
            else:
                title = no_extension_name + " (" + str(molecule_idx) + ")"

            # Create labfloor item.
            item = LabFloorItem(MoleculeConfiguration,
                                title=title,
                                tool_tip=comment,
                                molecule_idx=molecule_idx)

            # Add to result list.
            result.append(item)

        # Return the result list.
        return result

    def load(self, filename, molecule_idx=0):
        """
        Load the desired object in memory.

        @param filename : The path of the XYZ-file.
        @type           : string

        @return Desired object (MoleculeConfiguration)
        """
        # Read the file
        reader = XYZFileRawReader(filename)

        # Lists of elements and positions.
        elements = []
        positions = []

        # Loop over atoms.
        for atom in reader.atomList(molecule=molecule_idx):
            elements.append(SYMBOL_TO_ELEMENT[atom["element"]])
            positions.append(atom["coords"]*Angstrom)

        # Create configuration.
        configuration = MoleculeConfiguration(
            elements=elements,
            cartesian_coordinates=positions)

        return configuration
